#ifndef STREU_CPLUSLIB_OBJECT_H_INCLUDED
#define STREU_CPLUSLIB_OBJECT_H_INCLUDED

#include "cpluslib/config.h"
#if defined(STREU_CONFIG_OBJECT_HAS_TOSTRING)
#  include "cpluslib/string.h"
#endif

class Object;

/* reference counter for smart pointer */
struct ReferenceCount {
    std::size_t ptr_count;      // # of Ptrs referring here

    static ReferenceCount* MapAPointer(void* ptr) { return MapPointer(ptr); }
    static ReferenceCount* MapPointer(void* ptr);
#ifdef STREU_CONFIG_PTR_OPTIMIZE_OBJECT
    static inline ReferenceCount* MapPointer(Object* ptr);
#endif
    static ReferenceCount null_ref;
};

#ifdef STREU_CONFIG_PTR_OPTIMIZE_OBJECT
/* Object *with* reference count */
class Object {
 private:
    ReferenceCount rc;
    friend ReferenceCount* ReferenceCount::MapPointer(Object*);
 public:
    Object() { rc.ptr_count = 0; }
    virtual ~Object();
#ifdef STREU_CONFIG_OBJECT_HAS_TOSTRING
    virtual string_t toString();
#endif
};

inline ReferenceCount*
ReferenceCount::MapPointer(Object* ptr)
{
    return ptr ? &ptr->rc : &null_ref;
}
#else
/* Object without reference count */
class Object {
 public:
    virtual ~Object();
#ifdef STREU_CONFIG_OBJECT_HAS_TOSTRING
    virtual string_t toString();
#endif
};
#endif

#endif
